@@ -1,18 +1,7 @@
-from PIL import Image
-from dlib import cnn_face_detection_model_v1
-
-from controller import Emitter, Camera, GPS, Gyro
-from flockai.PyCatascopia.probelib.ProcessProbe import ProcessProbe
 from flockai.models.drones.autopilot_controlled_drone import AutopilotControlledDrone
 from flockai.models.drones.keyboard_controller_drone import KeyboardControlledDrone
 from flockai.models.probes.flockai_probe import FlockAIProbe
-from flockai.models.sensors.humidity import HumiditySensor
-from flockai.models.sensors.temperature import TemperatureSensor
-from flockai.utils.string_generator import StringGenerator
-from flockai.models.devices.device_enums import EnableableDevice, NonEnableableDevice, MotorDevice
-from flockai.utils.intensive_thread import IntensiveThread
-import numpy as np
-import json
+from flockai.models.devices.device_enums import EnableableDevice, NonEnableableDevice
 import time
 
 
@@ -32,56 +21,6 @@ class KeyboardMavic2DJI(KeyboardControlledDrone):
             self.probe.activate()
             print(f'{self.probe.get_name()} probe activated')
 
-    def send_msg(self, msg, emitter_devices: list):
-        """
-        Sends a message from the attached emitter device
-        :param emitter_devices: The emitter devices to send the message
-        :param msg: The message to send
-        :return: Time required to send the message
-        """
-        total_time = 0
-        probe_alive_time = self.probe.get_metric('ProbeAliveTimeMetric')
-
-        for emitter in emitter_devices:
-            if emitter not in self.devices:
-                print(f"The specified emitter device is not found: {emitter}")
-                return 0
-            f_emitter = self.devices[emitter]['device']
-            t1 = probe_alive_time.get_val()
-            message = json.dumps(msg)
-            f_emitter.send(message.encode('utf-8'))
-
-            t2 = probe_alive_time.get_val()
-            total_time += t2 - t1
-
-        return total_time
-
-    def receive_msgs(self, receiver_devices: list):
-        """
-        Receive messages on the receiver device
-        :param receiver_devices: The receiver device name as a string
-        :return: The list of messages and the time required to receive them
-        """
-        total_time = 0
-        messages = []
-        probe_alive_time = self.probe.get_metric('ProbeAliveTimeMetric')
-
-        for receiver in receiver_devices:
-            if receiver not in self.devices:
-                print(f"The specified receiver device is not found: {receiver}")
-                return [], 0
-
-            f_receiver = self.devices[receiver]['device']
-            while f_receiver.getQueueLength() > 0:
-                t1 = probe_alive_time.get_val()
-                message_received = f_receiver.getData().decode('utf-8')
-                messages.append(message_received)
-                f_receiver.nextPacket()
-                t2 = probe_alive_time.get_val()
-                total_time += t2 - t1
-
-        return messages, total_time
-
     def blink_led_lights(self, led_devices: list):
         """
         Blink led lights every 2 seconds
@@ -107,8 +46,10 @@ class KeyboardMavic2DJI(KeyboardControlledDrone):
 
         return self.energy_model.processing_energy.calculate(cpu_time_active=cpu_time, flight_time=flight_time, io_time=io_time)
 
-    def get_communication_energy(self, comm_emmition_time, comm_receival_time):
-        return self.energy_model.communication_energy.calculate(transmit_time=comm_emmition_time, receive_time=comm_receival_time, idle_time=0)
+    def get_communication_energy(self, comm_transmit_time, comm_receive_time):
+        probe_alive_time = self.probe.get_metric('ProbeAliveTimeMetric')
+        flight_time = probe_alive_time.get_val()
+        return self.energy_model.communication_energy.calculate(transmit_time=comm_transmit_time, receive_time=comm_receive_time, idle_time=flight_time-(comm_transmit_time + comm_receive_time))
 
     def run(self):
         # Wait a second before starting
@@ -127,10 +68,8 @@ class KeyboardMavic2DJI(KeyboardControlledDrone):
             if self.devices[device_name]['type'] == EnableableDevice.RECEIVER:
                 receiver_devices.append(device_name)
 
-        total_energy = 0
-        net_transaction_time = 0
-        comm_emmition_time = 0
-        comm_receival_time = 0
+        comm_transmit_time = 0
+        comm_receive_time = 0
 
         while self.step(self.basic_time_step) != -1:
             self.blink_led_lights(led_devices)
@@ -138,20 +77,31 @@ class KeyboardMavic2DJI(KeyboardControlledDrone):
 
             # Drone sends data specified by the user to the base station if ML is not running on board
             # TODO:
-            comm_emmition_time += self.send_msg(msg="test", emitter_devices=emitter_devices)
-            received_messages, receival_time = self.receive_msgs(receiver_devices=receiver_devices)
-            comm_receival_time += receival_time
+            t1 = time.time()
+            self.send_msg(msg="test", emitter_devices=emitter_devices)
+            t2 = time.time()
+            comm_transmit_time += t2 - t1
+            t1 = time.time()
+            received_messages = self.receive_msgs(receiver_devices=receiver_devices)
+            t2 = time.time()
+            comm_receive_time += t2 - t1
 
             # Execute prediction pipeline step
             if self.model is not None:
                 prediction = self.model.predict()
-                if prediction is not None:
-                    print(prediction)
+                # if prediction is not None:
+                #     print(prediction)
 
             # Get energy values on each cycle
-            print(self.get_processing_energy())
-            print(self.get_communication_energy(comm_emmition_time=comm_emmition_time, comm_receival_time=comm_receival_time))
+            energy = {**self.get_processing_energy(),
+                      **self.get_communication_energy(comm_transmit_time=comm_transmit_time, comm_receive_time=comm_receive_time)}
+            energy['total_energy'] = energy['e_proc'] + energy['e_comm']
 
+            # Update battery values
+            self.battery.remaining_energy_percentage = 1.0 - (energy['total_energy'] / self.battery.energy_capacity)
+
+            # Update log data every 5 minutes with energy values
+            print(energy, f'{self.battery.remaining_energy_percentage * 100}% battery remaining')
 
 
 class AutopilotMavic2DJI(AutopilotControlledDrone):
